/****************************************************************************
** Meta object code from reading C++ file 'kauthorized.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kconfig-6.25.0/src/core/kauthorized.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kauthorized.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11KAuthorizedE_t {};
} // unnamed namespace

template <> constexpr inline auto KAuthorized::qt_create_metaobjectdata<qt_meta_tag_ZN11KAuthorizedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KAuthorized",
        "authorize",
        "",
        "action",
        "GenericRestriction",
        "authorizeAction",
        "GenericAction",
        "authorizeControlModule",
        "pluginId",
        "SHELL_ACCESS",
        "GHNS",
        "LINEEDIT_REVEAL_PASSWORD",
        "LINEEDIT_TEXT_COMPLETION",
        "MOVABLE_TOOLBARS",
        "RUN_DESKTOP_FILES",
        "OPEN_WITH",
        "EDITFILETYPE",
        "OPTIONS_SHOW_TOOLBAR",
        "SWITCH_APPLICATION_LANGUAGE",
        "BOOKMARKS"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'authorize'
        QtMocHelpers::MethodData<bool(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 3 },
        }}),
        // Method 'authorize'
        QtMocHelpers::MethodData<bool(enum GenericRestriction)>(1, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 4, 3 },
        }}),
        // Method 'authorizeAction'
        QtMocHelpers::MethodData<bool(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 3 },
        }}),
        // Method 'authorizeAction'
        QtMocHelpers::MethodData<bool(enum GenericAction)>(5, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 6, 3 },
        }}),
        // Method 'authorizeControlModule'
        QtMocHelpers::MethodData<bool(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'GenericRestriction'
        QtMocHelpers::EnumData<enum GenericRestriction>(4, 4, QMC::EnumFlags{}).add({
            {    9, GenericRestriction::SHELL_ACCESS },
            {   10, GenericRestriction::GHNS },
            {   11, GenericRestriction::LINEEDIT_REVEAL_PASSWORD },
            {   12, GenericRestriction::LINEEDIT_TEXT_COMPLETION },
            {   13, GenericRestriction::MOVABLE_TOOLBARS },
            {   14, GenericRestriction::RUN_DESKTOP_FILES },
        }),
        // enum 'GenericAction'
        QtMocHelpers::EnumData<enum GenericAction>(6, 6, QMC::EnumFlags{}).add({
            {   15, GenericAction::OPEN_WITH },
            {   16, GenericAction::EDITFILETYPE },
            {   17, GenericAction::OPTIONS_SHOW_TOOLBAR },
            {   18, GenericAction::SWITCH_APPLICATION_LANGUAGE },
            {   19, GenericAction::BOOKMARKS },
        }),
    };
    return QtMocHelpers::metaObjectData<KAuthorized, qt_meta_tag_ZN11KAuthorizedE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KAuthorized::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KAuthorizedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KAuthorizedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11KAuthorizedE_t>.metaTypes,
    nullptr
} };

void KAuthorized::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KAuthorized *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->authorize((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->authorize((*reinterpret_cast<std::add_pointer_t<enum GenericRestriction>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->authorizeAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->authorizeAction((*reinterpret_cast<std::add_pointer_t<enum GenericAction>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->authorizeControlModule((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *KAuthorized::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KAuthorized::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KAuthorizedE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KAuthorized::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
