/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<kconfigpropertymap.h>)
#  include <kconfigpropertymap.h>
#endif
#if __has_include(<kwindowstatesaverquick.h>)
#  include <kwindowstatesaverquick.h>
#endif
#if __has_include(<types.h>)
#  include <types.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_config()
{
    qmlRegisterModule("org.kde.config", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<KAuthorizedForeign>("org.kde.config", 254);
    qmlRegisterEnum<KAuthorized::GenericRestriction>("KAuthorized::GenericRestriction");
    qmlRegisterEnum<KAuthorized::GenericAction>("KAuthorized::GenericAction");
    qmlRegisterTypesAndRevisions<KConfigPropertyMap>("org.kde.config", 254);
    qmlRegisterTypesAndRevisions<KConfigSkeletonForeign>("org.kde.config", 254);
    qmlRegisterTypesAndRevisions<KCoreConfigSkeletonForeign>("org.kde.config", 254);
    qmlRegisterEnum<KCoreConfigSkeleton::DisambiguateConstructor>("KCoreConfigSkeleton::DisambiguateConstructor");
    qmlRegisterTypesAndRevisions<KWindowStateSaverQuick>("org.kde.config", 254);
    QT_WARNING_POP
    qmlRegisterModule("org.kde.config", 254, 254);
}

static const QQmlModuleRegistration orgkdeconfigRegistration("org.kde.config", qml_register_types_org_kde_config);
